_msgs.cpm =
  [
    /* 0  */ "ERROR: Must specify the pin.",
    /* 1  */ "ERROR: Must specify the value.",
    /* 2  */ "ERROR: Invalid CP Value.",
    /* 3  */ "ERROR: Must specify the state.",
    /* 4  */ "ERROR: Invalid state.",
    /* 5  */ "ERROR: Must specify the group.",
    /* 6  */ "ERROR: Must specify the bit.",
    /* 7  */ "ERROR: Unknown CP.",
    /* 8  */ "ERROR: Failed to change the value for {}.",
    /* 9  */ "Changed the value for {} to {}.",
    /* 10 */ "ERROR: Failed to change the state for {}.",
    /* 11 */ "Changed the state for {} to {} with Assert {}.",
    /* 12 */ "ERROR: Failed to create the group {}.",
    /* 13 */ "Created the group {}.",
    /* 14 */ "ERROR: Failed to delete the group {}.",
    /* 15 */ "Deleted the group {}.",
    /* 16 */ "ERROR: Failed to change the state for {}",
    /* 17 */ "Changed the state for {} to {}.",
    /* 18 */ "ERROR: Failed to change the value for {}.",
    /* 19 */ "Changed the value for {} to {}.",
    /* 20 */ "ERROR: Failed to add {} to group {}.",
    /* 21 */ "Added {} to group {}.",
    /* 22 */ "ERROR: Failed to remove {} from group {}.",
    /* 23 */ "Removed {} from group {}.",
    /* 24 */ "ERROR: Invalid CP Group Value.",
  ];

function CpmStatusTableRender(xml)
{
    var i;
    var d = document.getElementById('cpmstatustable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No data is available for this CP.";
    }
    else
    {
        var err = GetXmlData(xml, "err");

        if (err)
        {
            r += err;
        }
        else
        {
            var type = GetXmlData(xml, "t");

            r += "<table class=\"basicTable cpmSingleTable\">";

            r += "<tr><td><b>Name</b><td colspan=\"32\">" + GetXmlData(xml, "n") + "</td></tr>";
            r += "<tr><td><b>State</b><td colspan=\"32\">" + GetXmlData(xml, "s") + "</td></tr>";
            if (type)
            {
                r += "<tr><td><b>Type</b><td colspan=\"32\">" + type + "</td></tr>";
            }
            r += "<tr><td><b>Value</b><td colspan=\"32\">" + GetXmlData(xml, "e") + "</td></tr>";

            r += "<tr><td><b>Bit</b></td>"
            for (i = 31; i >= 0; i--)
            {
                r += "<td><b>";
                if (i > 9)
                {
                   r += parseInt(i / 10) + "<br/>" + (i % 10);
                }
                else
                {
                   r += i;
                }
                r += "</b></td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Level</b></td>"
            for (i = 31; i >= 0; i--)
            {
            var level = GetXmlData(xml, "v" + i);
                r += "<td>";
                r += level ? level : " ";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>I/O</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var io = GetXmlData(xml, "io" + i);
                r += "<td>";
                r += io ? io : " ";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Logic</b></td>"
            for (i = 31; i >= 0; i--)
            {
                r += "<td>";
                r += (GetXmlData(xml, "l" + i)) ? "I" : "";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Binary</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var state = GetXmlData(xml, "s" + i);
                r += "<td>";
                r += (state) ? state : "x";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>CP#</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var pin = GetXmlData(xml, "sp" + i);
                r += "<td>";
                if (pin)
                {
                    pin++;

                    if (pin > 9)
                    {
                       r += parseInt(pin / 10) + "<br/>" + (pin % 10);
                    }
                    else
                    {
                       r += pin;
                    }
                }
                r += "</td>";
            }
            r += "</tr>";

            if (type)
            {
                r += "<tr><td><b>Groups</b></td><td colspan=\"32\">"
                for (i = 31; i >= 0; i--)
                {
                    var group = GetXmlData(xml, "mg" + i);
                    if (group)
                    {
                        r += group + "<br/>";
                    }
                }
                r += "</tr>";
            }

            r += "</table>";
        }
    }

    d.innerHTML = r;
}

function CpmPinsTableRender(xml)
{
    var d = document.getElementById('cpmpinstable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No CP data is available.";
    }
    else
    {
        r += "<table class=\"basicTable cpmMultipleTable\"><thead><tr><th>CP</th><th>Pin #</th><th>Configured As</b></th><th>State</th><th>Groups</th><th>Active In Group</th></tr></thead><tbody>";

        for (var i = 0; xml && i < xml.childNodes.length; i++)
        {
            var node = xml.childNodes[i];

            if (node.nodeName.toLowerCase() == "c")
            {
                var cp = GetXmlData(node, "n");

                r += "<tr>";
                r += "<td><a href=\"javascript:void(0)\" onclick=\"ajax_AddToHistory('CpmPinsPage', [true, 'pin=" + cp + "']); return false;\">" + cp + "</a></td>";
                r += "<td>" + GetXmlData(node, "p") + "</td>";
                r += "<td>" + GetXmlData(node, "t") + "</td>";
                r += "<td>" + GetXmlData(node, "s") + "</td>";
                r += "<td>" + GetXmlData(node, "g") + "</td>";
                var active = GetXmlData(node, "a");
                r += "<td>";
                r += (active) ? active : "&lt;available&gt;";
                r += "</td>";
                r += "</tr>";
            }
        }

        r += "</tbody></table>";
    }

    d.innerHTML = r;
}

function CpmPinsFormRenderSelectPin(xml)
{
    var r = '';

    r += "<select name=\"pin\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var node = xml.childNodes[i];

        r += "<option value=\"" + node.nodeName + "\"";
        r += (node.firstChild.nodeValue == "1") ? " selected" : "";
        r += ">" + node.nodeName + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmPinsFormRender(xml)
{
    var d;

    if (!xml || !xml.hasChildNodes())
    {
        d = "Failed to generate form.";
    }
    else
    {
        d = CpmPinsFormRenderSelectPin(xml);
    }

    document.getElementById('cpmpinvalue').innerHTML = d;
    document.getElementById('cpmpindirection').innerHTML = d;
}

function CpmGroupsTableRender(xml)
{
    var d = document.getElementById('cpmgroupstable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No CP data is available.";
    }
    else
    {
        r += "<table class=\"basicTable cpmMultipleTable\"><thead><tr><th>Group Name</th><th>State</th><th>CP Info</th></tr></thead><tbody>";

        for (var i = 0; xml && i < xml.childNodes.length; i++)
        {
            var node = xml.childNodes[i];

            if (node.nodeName.toLowerCase() == "g")
            {
                var group = GetXmlData(node, "n");

                r += "<tr>";
                r += "<td><a href=\"javascript:void(0)\" onclick=\"return ajax_AddToHistory('CpmGroupsPage', [true, 'group=" + encodeURIComponent(group) + "']);\">" + group + "</a></td>";
                r += "<td>" + GetXmlData(node, "s") + "</td>";
                r += "<td>" + GetXmlData(node, "c") + "</td>";
                r += "</tr>";
            }
        }

        r += "</tbody></table>";
    }

    d.innerHTML = r;
}

function CpmGroupsFormRenderSelectGroup(xml)
{
    var r = '';

    r += "<select name=\"group\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var g = xml.getElementsByTagName("g" + i);
        if (!g || !g.length)
        {
            break;
        }
        var gv = g[0].firstChild.nodeValue;

        var gs = xml.getElementsByTagName("gs" + i);
        var gsv = gs[0].firstChild.nodeValue;

        r += "<option value=\"" + gv + "\"";
        r += (gsv == "1") ? " selected" : "";
        r += ">" + gv + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmGroupsFormRenderSelectPin(xml)
{
    var r = '';

    r += "<select name=\"pin\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var cp = xml.getElementsByTagName("CP" + (i + 1));
        if (!cp || !cp.length)
        {
            break;
        }
        var cpv = cp[0].nodeName;

        r += "<option value=\"" + cpv + "\">" + cpv + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmGroupsFormRender(xml)
{
    var g;
    var p;
    var b;

    if (!xml || !xml.hasChildNodes())
    {
        g = "Failed to generate form.";
        p = g;
    }
    else
    {
        g = CpmGroupsFormRenderSelectGroup(xml);
        p = CpmGroupsFormRenderSelectPin(xml);
    }

    {
        b = "<select name=\"bit\">";
        b += "<option value=\"Next\">Next</option>";
        for (var i = 0; i < 32; ++i)
        {
            b += "<option value=\"" + i + "\">" + i + "</option>";
        }
        b += "</select>";
    }

    document.getElementById('cpmgroupdeleteformselectgroup').innerHTML = g;

    document.getElementById('cpmgroupstateformselectgroup').innerHTML = g;

    document.getElementById('cpmgroupsetformselectgroup').innerHTML = g;

    document.getElementById('cpmgroupaddformselectpin').innerHTML = p;
    document.getElementById('cpmgroupaddformselectgroup').innerHTML = g;
    document.getElementById('cpmgroupaddformselectbit').innerHTML = b;

    document.getElementById('cpmgroupremoveformselectpin').innerHTML = p;
    document.getElementById('cpmgroupremoveformselectgroup').innerHTML = g;
}

function SetCpmNav(xml)
{
    SetNav(xml, false, true);
}
